@@ -1,7 +1,7 @@
 #' @inheritParams bcov.test
 #' @rdname bcov
 #' @return 
-#' \item{\code{bcor }}{ sample version of ball correlation.}
+#' \item{\code{bcor }}{ Ball Correlation statistic.}
 #' @export
 #' @examples
 #' ############# Ball Correlation #############
@@ -9,17 +9,15 @@
 #' x <- 1:num
 #' y <- 1:num
 #' bcor(x, y)
-#' bcor(x, y, weight = TRUE)
 #' bcor(x, y, weight = "prob")
 #' bcor(x, y, weight = "chisq")
 bcor <- function(x, y, distance = FALSE, weight = FALSE) {
+  weight <- examine_weight_arguments(weight)
   x <- as.matrix(x)
   y <- as.matrix(y)
   x_y_info <- examine_x_y(x, y)
   p <- x_y_info[2]
   #
-  weight <- examine_weight_arguments(weight)
-  #
   if(distance == FALSE) {
     if(p != 1) {
       x <- as.double(as.vector(as.matrix(dist(x, diag = TRUE))))
@@ -38,7 +36,6 @@ bcor <- function(x, y, distance = FALSE, weight = FALSE) {
     dst_y <- as.integer(1)
     dst_x <- as.integer(1)
   }
-  #
   bcor_stat <- as.double(numeric(3))
   x_number <- as.integer(1)
   f_number <- as.integer(1)
@@ -47,7 +44,6 @@ bcor <- function(x, y, distance = FALSE, weight = FALSE) {
   p <- as.integer(1)
   k <- as.integer(1)
   nth <- as.integer(1)
-  #
   res <- .C("bcor_test", bcor_stat, y, x, x_number, f_number, size_number, num, p, k, dst_y, dst_x, nth)
   bcor_stat <- res[[1]]
   bcor_stat <- select_ball_stat(bcor_stat, weight, type = "bcor")
@@ -55,72 +51,70 @@ bcor <- function(x, y, distance = FALSE, weight = FALSE) {
 }
 
 
-#' @title Ball Correlation Sure Independence Screening
-#' @author Wenliang Pan, Weinan Xiao, Xueqin Wang, Hongtu Zhu
-#' @description Generic non-parametric sure independence screening procedure based on ball correlation.
+#' @title Ball Correlation based Sure Independence Screening (BCor-SIS)
+#' @author Wenliang Pan, Weinan Xiao, Xueqin Wang, Hongtu Zhu, Jin Zhu
+#' @description Generic non-parametric sure independence screening (SIS) procedure based on Ball Correlation.
 #' Ball correlation is a generic multivariate measure of dependence in Banach space.
 #' @inheritParams bcov.test
 #' @param x a numeric matirx or data.frame included \eqn{n} rows and \eqn{p} columns. 
 #' Each row is an observation vector and each column corresponding to a explanatory variable, generally \eqn{p >> n}.
 #' @param d the hard cutoff rule suggests selecting \eqn{d} variables. Setting \code{d = "large"} or 
-#' \code{ d = "small"} means \code{n-1} or \code{floor(n/log(n))} 
-#' variables are selected. If \code{d} is a integer, 
-#' \code{d} variables are selected. Default: \code{d = "small"}
-#' @param weight when \code{weight = TRUE}, weighted ball correlation is used instead of ball correlation. Default: \code{ weight = FALSE}  
-#' @param method method for sure independence screening procedure, include: \code{"standard"},
-#' \code{"lm"}, \code{"gam"}, \code{"interaction"} and \code{"survival"}.
-#' Setting \code{method = "standard"} means standard sure independence screening procedure 
-#' based on ball correlation while options
-#' \code{"lm"} and \code{"gam"} carry out iterative BCor-SIS procedure with ordinary 
+#' \code{ d = "small"} means \code{n - 1} or \code{floor(n/log(n))} 
+#' variables are selected. If \code{d} is a integer, \code{d} variables are selected. Default: \code{d = "small"}.
+#' @param method specific method for the BCor-SIS procedure. It must be one of \code{"standard"},
+#' \code{"lm"}, \code{"gam"}, \code{"interaction"}, or \code{"survival"}.
+#' Setting \code{method = "standard"} means performing standard SIS procedure 
+#' while the options \code{"lm"} and \code{"gam"} mean carrying out iterative SIS procedure with ordinary 
 #' linear regression and generalized additive models, respectively.
-#' Options \code{"interaction"} and \code{"survival"} are designed for detecting variables 
-#' with potential linear interaction or associated with censored responses. Default: \code{method = "standard"}
+#' The options \code{"interaction"} and \code{"survival"} are designed for detecting variables 
+#' with potential linear interaction and associated with left censored responses, respectively. 
+#' Any unambiguous substring can be given. Default: \code{method = "standard"}.
 #' @param distance if \code{distance = TRUE}, \code{y} will be considered as a distance matrix. 
-#' Arguments only available when \code{ method = "standard"} and \code{ method = "interaction"}. Default: \code{distance = FALSE}
+#' Arguments only available when \code{method = "standard"} and \code{method = "interaction"}. Default: \code{distance = FALSE}.
 #' @param parms parameters list only available when \code{method = "lm"} or \code{"gam"}. 
 #' It contains three parameters: \code{d1}, \code{d2}, and \code{df}. \code{d1} is the
-#' number of initially selected variables, \code{d2} is the number of variables collection size added in each iteration.
-#' \code{df} is degree freedom of basis in generalized additive models 
-#' playing a role only when \code{method = "gam"}. Default: \code{ parms = list(d1 = 5, d2 = 5, df = 3)}
+#' number of initially selected variables, \code{d2} is the number of variables added in each iteration.
+#' \code{df} is a degree freedom of basis in generalized additive models playing a role only when \code{method = "gam"}. 
+#' Default: \code{parms = list(d1 = 5, d2 = 5, df = 3)}.
 #' 
 #' @return 
-#' \item{\code{ix }}{ the vector of indices selected by ball correlation sure independence screening procedure.} 
+#' \item{\code{ix }}{ the indices vector corresponding to variables selected by BCor-SIS.} 
 #' \item{\code{method }}{ the method used.} 
 #' \item{\code{weight }}{ the weight used.} 
-#' \item{\code{complete.info }}{ a \code{list} containing at least one \eqn{p x 3} matrix, where each row is corresponding to variable and each column is corresponding to differe ball correlation weight. If \code{method = "gam"} or \code{method = "lm"}, \code{complete.info} is empty list.} 
+#' \item{\code{complete.info }}{ a \code{list} mainly containing a \eqn{p \times 3} matrix, 
+#' where each row is a variable and each column is a weight Ball Correlation statistic. 
+#' If \code{method = "gam"} or \code{method = "lm"}, \code{complete.info} is an empty list.} 
 #' 
 #' @details 
-#' \code{bcorsis} implements a model-free generic screening procedure, 
-#' BCor-SIS, with fewer and less restrictive assumptions. 
-#' The sample sizes (number of rows or length of the vector) of the 
-#' two variables \code{x} and \code{y} must agree, 
-#' and samples must not contain missing values. 
-#' 
-#' BCor-SIS procedure for censored response is carried out when \code{method = "survival"}. At that time, 
-#' the matrix or data.frame pass to argument \code{y} must have exactly two columns and the first column is 
-#' event (failure) time while the second column is censored status, a dichotomous variable. 
-#' 
-#' If we set \code{distance = TRUE}, arguments \code{y} is considered as distance matrix, 
-#' otherwise \code{y} is treated as data.
-#' 
-#' BCor-SIS is based on a recently developed universal dependence measure: Ball correlation (BCor). 
-#' BCor efficiently measures the dependence between two random vectors, which is between 
-#' 0 and 1, and 0 if and only if these two random vectors are independent under some mild conditions.
-#' (See the manual page for \code{\link{bcor}}.)
+#' \code{bcorsis} performs a model-free generic sure independence screening procedure, 
+#' BCor-SIS, to pick out variables from \code{x} which are potentially associated with \code{y}. 
+#' BCor-SIS relies on Ball correlation, a universal dependence measure in Banach spaces.
+#' Ball correlation (BCor) ranges from 0 to 1. A larger BCor implies they are likely to be associated while 
+#' Bcor is equal to 0 implies they are unassociated. (See \code{\link{bcor}} for details.)
+#' Consequently, BCor-SIS pick out variables with larger Bcor values with \code{y}.
 #' 
 #' Theory and numerical result indicate that BCor-SIS has following advantages:
+#' \itemize{
+#' \item BCor-SIS can retain the efficient variables even when the dimensionality (i.e., \code{ncol(x)}) is 
+#' an exponential order of the sample size (i.e., \code{exp(nrow(x))});
+#' \item It is distribution-free and model-free;
+#' \item It is very robust;
+#' \item It is works well for complex data, such as shape and survival data;
+#' }
 #' 
-#' (i) It has a strong screening consistency property without finite sub-exponential moments of the data.
-#' Consequently, even when the dimensionality is an exponential order of the sample size, BCor-SIS still 
-#' almost surely able to retain the efficient variables.
+#' If \code{x} is a matrix, the sample sizes of \code{x} and \code{y} must agree.
+#' If \code{x} is a \code{\link{list}} object, each element of this \code{list} must with the same sample size.
+#' \code{x} and \code{y} must not contain missing or infinite values. 
 #' 
-#' (ii) It is nonparametric and has the property of robustness.
+#' When \code{method = "survival"}, the matrix or data.frame pass to \code{y} must have exactly two columns, where the first column is 
+#' event (failure) time while the second column is a dichotomous censored status.
 #' 
-#' (iii) It works well for complex responses and/or predictors, such as shape or survival data
+#' @note 
+#' \code{bcorsis} simultaneously computing Ball Correlation statistics with 
+#' \code{"constant"}, \code{"probability"}, and \code{"chisquare"} weights.
+#' Users can get other Ball Correlation statistics with different weight in the \code{complete.info} element of output. 
+#' We give a quick example below to illustrate. 
 #' 
-#' (iv) It can extract important features even when the underlying model is complicated.
-#' 
-#'   
 #' @seealso 
 #' \code{\link{bcor}}
 #' 
@@ -137,9 +131,10 @@ bcor <- function(x, y, distance = FALSE, weight = FALSE) {
 #' p <- 3000
 #' x <- matrix(rnorm(n * p), nrow = n)
 #' error <- rnorm(n)
-#' y <- 3*x[, 1] + 5*(x[, 3])^2 + error
+#' y <- 3 * x[, 1] + 5 * (x[, 3])^2 + error
 #' res <- bcorsis(y = y, x = x)
 #' head(res[["ix"]])
+#' head(res[["complete.info"]])
 #' 
 #' ############### BCor-SIS: Censored Data Example ###############
 #' data("genlung")
@@ -156,7 +151,7 @@ bcor <- function(x, y, distance = FALSE, weight = FALSE) {
 #' p <- 3000
 #' x <- matrix(rnorm(n * p), nrow = n)
 #' error <- rnorm(n)
-#' y <- 3*x[, 1]*x[, 5]*x[, 10] + error
+#' y <- 3 * x[, 1] * x[, 5] * x[, 10] + error
 #' res <- bcorsis(y = y, x = x, method = "interaction")
 #' head(res[["ix"]])
 #' 
@@ -170,7 +165,7 @@ bcor <- function(x, y, distance = FALSE, weight = FALSE) {
 #' x <- rmvnorm(n = n, sigma = sigma_mat)
 #' error <- rnorm(n)
 #' rm(sigma_mat); gc(reset = TRUE)
-#' y <- 3*(x[, 1])^2 + 5*(x[, 2])^2 + 5*x[, 8] - 8*x[, 16] + error
+#' y <- 3 * (x[, 1])^2 + 5 * (x[, 2])^2 + 5 * x[, 8] - 8 * x[, 16] + error
 #' res <- bcorsis(y = y, x = x, method = "lm", d = 15)
 #' res <- bcorsis(y = y, x = x, method = "gam", d = 15)
 #' res[["ix"]]
@@ -181,19 +176,19 @@ bcor <- function(x, y, distance = FALSE, weight = FALSE) {
 #' p <- 3000
 #' x <- matrix(rnorm(n * p), nrow = n)
 #' error <- rnorm(n)
-#' y <- 3*x[, 1] + 5*(x[, 3])^2 + error
+#' y <- 3 * x[, 1] + 5 * (x[, 3])^2 + error
 #' res <- bcorsis(y = y, x = x, weight = "prob")
 #' head(res[["ix"]])
 #' # Alternative, chisq weight:
 #' res <- bcorsis(y = y, x = x, weight = "chisq")
 #' head(res[["ix"]])
 #' }
-bcorsis <- function(x, y, d = "small", weight = FALSE, 
+bcorsis <- function(x, y, d = "small", weight = c("constant", "probability", "chisquare"), 
                     method = "standard", distance = FALSE,
                     parms = list(d1 = 5, d2 = 5, df = 3),
-                    num.threads = 2)
+                    num.threads = 0)
 {
-  seed <- 4
+  seed <- 1
   y <- as.matrix(y)
   x <- as.matrix(x)
   n <- examine_x_y(x, y)[1]
@@ -237,11 +232,11 @@ bcorsis <- function(x, y, d = "small", weight = FALSE,
     # data prepare for screening:
     if(distance == FALSE) {
       if(y_p != 1) {
-        y <- as.vector(as.matrix(dist(y, diag = TRUE)))
+        y <- as.vector(dist(y))
         distance <- TRUE
       }
     } else {
-      y <- as.vector(y)
+      y <- y[lower.tri(y)]
     }
     # BCor-SIS:
     rcory_result <- apply_bcor_wrap(x = x, y = y, n = n, p = p, 
@@ -308,13 +303,14 @@ bcorsis <- function(x, y, d = "small", weight = FALSE,
           dat <- as.data.frame(cbind(x[, index], lastpickout_dat))
           colnames(dat)[1] <- colnames(x)[index]
           # colnames(dat) <- paste0("x",c(x,Xhavepickout))
-          gam::gam(formula_one, data = dat)[["residuals"]]
+          suppressWarnings(residuals_value <- gam::gam(formula_one, data = dat)[["residuals"]])
+          residuals_value
         })
         
         # gam fit for y
         dat <- data.frame("y" = y, lastpickout_dat)
         formula_Y <- as.formula(paste("y ~ ", lastpickout_formula))
-        y <- gam::gam(formula = formula_Y,data = dat)$residuals
+        suppressWarnings(y <- gam::gam(formula = formula_Y, data = dat)$residuals)
         
         # BCor-screening
         y_copy <- preprocess_bcorsis_y(y, y_p)[[1]]
@@ -337,7 +333,7 @@ bcorsis <- function(x, y, d = "small", weight = FALSE,
 #' @title Ball Correlation Sure Independence Screening For Survival data
 #' @description Utilize extension of Ball Correlation in survival to select d variables related to survival status.
 #' @inheritParams bcorsis
-#' @param y a numeric matirx(first column should be event time, second column should be survival status) or Surv object
+#' @param y a numeric matrix (first column should be event time, second column should be survival status) or Surv object
 #' @param standized allows the user to standardize the covariate
 #' @return the ids of selected variables
 #' @noRd
