@@ -1,54 +1,61 @@
 #' @title Ball Covariance Test
 #' @author Wenliang Pan, Xueqin Wang, Heping Zhang, Hongtu Zhu, Jin Zhu
-#' @description Ball covariance test of multivariate independence. 
-#' Ball covariance are generic multivariate measures of dependence in Banach space.
+#' @description Ball Covariance test of independence. 
+#' Ball covariance are generic dependence measures in Banach spaces.
 #' 
 #' @inheritParams bd.test
-#' @param x a numeric vector, matirx, data.frame or \code{dist} object or list containing numeric vector, matrix, data.frame, or \code{dist} object.
-#' @param y a numeric vector, matirx, data.frame or \code{dist} object.
-#' @param distance if \code{distance = TRUE}, \code{x} and \code{y} will be considered as a distance matrix. Default: \code{distance = FALSE}
-#' @param weight a logical or character value used to choose the form of weight. 
-#' If \code{weight = FALSE}, the ball covariance / correlation with constant weight is used. 
-#' Alternatively, \code{weight = TRUE} and \code{weight = "prob"} indicates the probability weight is chosen 
-#' while setting \code{weight = "chisq"} means select the Chi-square weight. 
-#' Note that this arguments actually only influences the printed result in R console and is 
-#' only available for the \code{bcov.test} function at present. Default: \code{weight = FALSE}
-## @param type If \code{type = 'bcor'}, ball correlation will be used instead of ball covariance.(default \code{type = 'bcov'})
+#' @param x a numeric vector, matrix, data.frame, or a list containing at least two numeric vectors, matrices, or data.frames.
+#' @param y a numeric vector, matrix, or data.frame.
+#' @param num.permutations the number of permutation replications. When \code{num.permutations = 0}, the function just returns
+#' the Ball Covariance statistic. Default: \code{num.permutations = 99}.
+#' @param distance if \code{distance = TRUE}, the elements of \code{x} and \code{y} are considered as distance matrices.
+#' @param weight a logical or character string used to choose the weight form of Ball Covariance statistic.. 
+#' If input is a character string, it must be one of \code{"constant"}, \code{"probability"}, or \code{"chisquare"}. 
+#' Any unambiguous substring can be given. 
+#' If input is a logical value, it is equivalent to \code{weight = "probability"} if \code{weight = TRUE} while 
+#' equivalent to \code{weight = "constant"} if \code{weight = FALSE}.
+#' Default: \code{weight = FALSE}.
+## @param type If \code{type = 'bcor'}, Ball Correlation will be used instead of Ball Covariance.(default \code{type = 'bcov'})
 ## @param method if \code{method = 'permute'}, a permutation procedure will be carried out;
 ## if \code{method = 'approx'}, the p-values based on approximate Ball Covariance distribution are given.(Test arguments)
-#' @param num.threads Number of threads. Default \code{num.threads = 1}.
 #' 
-#' @return bcov.test returns a list with class "htest" containing the following components:
-#' \item{\code{statistic}}{ball covariance or ball correlation statistic.}            
+#' @return If \code{num.permutations > 0}, \code{bcov.test} returns a \code{htest} class object containing the following components:
+#' \item{\code{statistic}}{Ball Covariance statistic.}            
 #' \item{\code{p.value}}{the p-value for the test.}  
 #' \item{\code{replicates}}{permutation replications of the test statistic.}
 #' \item{\code{size}}{sample size.} 
-#' \item{\code{complete.info}}{a \code{list} containing multiple statistics value and their corresponding $p$ value.}
+#' \item{\code{complete.info}}{a \code{list} mainly containing two vectors, the first vector is the Ball Covariance statistics 
+#' with different weights, the second is the \eqn{p}-values of weighted Ball Covariance tests.}
 #' \item{\code{alternative}}{a character string describing the alternative hypothesis.}
 #' \item{\code{method}}{a character string indicating what type of test was performed.}
 #' \item{\code{data.name}}{description of data.}
+#' If \code{num.permutations = 0}, \code{bcov.test} returns a statistic value.
 #' 
 #' @details 
-#' \code{bcov.test} are non-parametric tests of multivariate independence in Banach space. 
-#' The test decision is obtained via permutation, with \code{num.permutations} replicates.
+#' \code{bcov.test} are non-parametric tests of independence in Banach spaces. 
+#' It can detect the dependence between two random objects (variables) and 
+#' the mutual dependence among at least three random objects (variables).
 #' 
 #' If two samples are pass to arguments \code{x} and \code{y}, the sample sizes (i.e. number of rows or length of the vector) 
-#' of the two variables must agree. If a \code{\link{list}} object is passed to \code{x}, 
-#' each element must with same sample sizes. Moreover, data pass to \code{x} or \code{y} 
+#' of the two variables must agree. If a \code{\link{list}} object is passed to \code{x}, this \code{list} must contain at least
+#' two numeric vectors, matrices, or data.frames, and each element of this \code{list} 
+#' must with the same sample size. Moreover, data pass to \code{x} or \code{y} 
 #' must not contain missing or infinite values. 
-#' If we set \code{distance = TRUE}, arguments \code{x}, \code{y} can be a \code{dist} object or a
-#' symmetric numeric matrix recording distance between samples; 
-#' otherwise, these arguments are treated as data.
+#' If \code{distance = TRUE}, \code{x} is considered as a distance matrix or a list containing distance matrices, 
+#' and \code{y} is considered as a distance matrix; otherwise, these arguments are treated as data.
 #' 
-#' The \code{bcov.test} statistic is \code{bcov} or \code{ bcor} which are dependence measure 
-#' in Banach space. The \code{bcor} test statistic is based on the normalized 
-#' coefficient of ball covariance. (See the manual page for \code{\link{bcov}} or \code{\link{bcor}}.)
+#' \code{bcov.test} utilizes the Ball Covariance statistics (see \code{\link{bcov}}) to measure dependence and 
+#' derives a \eqn{p}-value via replicating the random permutation \code{num.permutations} times.
 #' 
-#' For the general problem of testing independence when the distributions of \eqn{X} and 
-#' \eqn{Y} are unknown, the test based on \code{bcov} can be implemented as a permutation test.
-#' See (Jin et al 2018) for theoretical properties of the test, including statistical consistency.
+#' See Pan et al 2018 for theoretical properties of the test, including statistical consistency.
 #' 
-#' @references Jin, Zhu, Wenliang Pan, Wei Zheng, and Xueqin Wang (2018). Ball: An R package for detecting distribution difference and association in metric spaces. arXiv preprint arXiv:1811.03750. URL http://arxiv.org/abs/1811.03750.
+#' @note Actually, \code{bcov.test} simultaneously computing Ball Covariance statistics with 
+#' \code{"constant"}, \code{"probability"}, and \code{"chisquare"} weights.
+#' Users can get other Ball Covariance statistics with different weight and their corresponding \eqn{p}-values 
+#' in the \code{complete.info} element of output. We give a quick example below to illustrate. 
+#' 
+#' @references Wenliang Pan, Xueqin Wang, Heping Zhang, Hongtu Zhu & Jin Zhu (2019) Ball Covariance: A Generic Measure of Dependence in Banach Space, Journal of the American Statistical Association, DOI: 10.1080/01621459.2018.1543600
+#' @references Jin Zhu, Wenliang Pan, Wei Zheng, and Xueqin Wang (2018). Ball: An R package for detecting distribution difference and association in metric spaces. arXiv preprint arXiv:1811.03750. http://arxiv.org/abs/1811.03750
 #' 
 #' @rdname bcov.test
 #' 
@@ -63,13 +70,18 @@
 #' x <- runif(50, 0, 4*pi)
 #' y <- cos(x) + error
 #' # plot(x, y)
-#' bcov.test(x, y)
+#' res <- bcov.test(x, y)
+#' res
+#' ## get all Ball Covariance statistics:
+#' res[["complete.info"]][["statistic"]]
+#' ## get all test result:
+#' res[["complete.info"]][["p.value"]]
 #' 
 #' ################# Quick Start #################
 #' x <- matrix(runif(50 * 2, -pi, pi), nrow = 50, ncol = 2)
-#' error <- runif(50, min = -0.3, max = 0.3)
-#' y <- (sin((x[,1])^2 + x[,2])) + error
-#' bcov.test(x = x, y = y)
+#' error <- runif(50, min = -0.1, max = 0.1)
+#' y <- sin(x[,1] + x[,2]) + error
+#' bcov.test(x = x, y = y, weight = "prob")
 #' 
 #' ################# Ball Covariance Test for Non-Hilbert Data #################
 #' # load data:
@@ -86,14 +98,19 @@
 #' Dy <- nhdist(ArcticLake[["x"]], method = "compositional")
 #' Dx <- dist(ArcticLake[["depth"]])
 #' # hypothesis test with weighted BCov:
-#' bcov.test(x = Dx, y = Dy, distance = TRUE, weight = TRUE)
+#' bcov.test(x = Dx, y = Dy, distance = TRUE, weight = "prob")
 #' 
 #' ################# Mutual Independence Test #################
-#' x <- rnorm(30)
-#' y <- (x > 0) * x + rnorm(30)
-#' z <- (x <= 0) * x + rnorm(30)
+#' x <- rnorm(50)
+#' y <- (x > 0) * x + rnorm(50)
+#' z <- (x <= 0) * x + rnorm(50)
 #' data_list <- list(x, y, z)
 #' bcov.test(data_list)
+#' data_list <- lapply(data_list, function(x) {
+#'   as.matrix(dist(x))
+#' })
+#' bcov.test(data_list, distance = TRUE)
+#' bcov.test(data_list, distance = FALSE, weight = "chi")
 #' 
 #' ################# Mutual Independence Test for Meteorology data #################
 #' data("meteorology")
@@ -106,7 +123,7 @@ bcov.test <- function(x, ...) UseMethod("bcov.test")
 #' @method bcov.test default
 bcov.test.default <- function(x, y = NULL, num.permutations = 99, 
                               distance = FALSE, weight = FALSE, 
-                              seed = 4, num.threads = 1, ...)
+                              seed = 1, num.threads = 0, ...)
 {
   method = 'permute'
   data_name <- paste(deparse(substitute(x)), "and", deparse(substitute(y)))
@@ -126,18 +143,18 @@ bcov.test.default <- function(x, y = NULL, num.permutations = 99,
   if(num.permutations == 0) {
     return(result)
   } else {
-    if (weight == "none") {
+    if (weight == "constant") {
       stat <- result[["statistic"]][1]
       pvalue <- result[["p.value"]][1]
-      weight_name <- "Constant"
-    } else if (weight == "prob") {
+      weight_name <- "constant"
+    } else if (weight == "probability") {
       stat <- result[["statistic"]][2]
       pvalue <- result[["p.value"]][2]
-      weight_name <- "Probability"
-    } else {
+      weight_name <- "probability"
+    } else if (weight == "chisquare") {
       stat <- result[["statistic"]][3]
       pvalue <- result[["p.value"]][3]
-      weight_name <- "Chisquare"
+      weight_name <- "chisquare"
     } 
     
     data_name <- paste0(data_name,"\nnumber of observations = ", result[["info"]][["N"]])
@@ -185,6 +202,8 @@ bcov.test.default <- function(x, y = NULL, num.permutations = 99,
 #' ################  Formula interface  ################
 #' ## independence test:
 #' bcov.test(~ CONT + INTG, data = USJudgeRatings)
+#' ## independence test with chisquare weight:
+#' bcov.test(~ CONT + INTG, data = USJudgeRatings, weight = "chi")
 #' ## mutual independence test:
 #' bcov.test(~ CONT + INTG + DMNR, data = USJudgeRatings)
 bcov.test.formula <- function(formula, data, subset, na.action, ...) {
@@ -219,24 +238,42 @@ bcov.test.formula <- function(formula, data, subset, na.action, ...) {
 bcov_test_internal <- function(x, y, num.permutations = 99, distance = FALSE, weight = FALSE, 
                                seed = 4, method = 'permute', num.threads)
 {
-  x <- as.matrix(x)
-  y <- as.matrix(y)
-  x_y_info <- examine_x_y(x, y)
-  num <- x_y_info[1]
-  p <- x_y_info[2]
-  #
-  if(distance == FALSE) {
-    if(p != 1) {
-      x <- as.vector(as.matrix(dist(x, diag = TRUE)))
-      y <- as.vector(as.matrix(dist(y, diag = TRUE)))
-      distance <- TRUE
+  if (class(x) == "dist" && class(y) == "dist") {
+    distance <- TRUE
+  }
+  if (distance) {
+    if (class(x) == "dist" || class(y) == "dist") {
+      if (class(x) != "dist") {
+        x <- as.dist(x)
+      }
+      if (class(y) != "dist") {
+        y <- as.dist(y)
+      }
+      num <- attr(x, "Size")
+      x <- as.vector(x)
+      y <- as.vector(y)
+    } else {
+      num <- nrow(x)
+      x <- x[lower.tri(x)]
+      y <- y[lower.tri(y)]
     }
   } else {
-    x <- as.vector(x)
-    y <- as.vector(y)
+    x <- as.matrix(x)
+    y <- as.matrix(y)
+    num <- nrow(x)
+    if (ncol(x) != 1 || ncol(y) != 1) {
+      x <- as.vector(dist(x))
+      y <- as.vector(dist(y))
+      distance <- TRUE
+    } else {
+      x <- as.vector(x)
+      y <- as.vector(y)
+    }
   }
+  examine_x_y_bcov(x, y)
+  
   ## memory protect step:
-  memoryAvailable(num, funs = 'BI.test')
+  # memoryAvailable(num, funs = 'BI.test')
   ## examine test type:
   # type <- examine_type_arguments(type)
   ## examine num.permutations arguments:
@@ -253,17 +290,17 @@ bcov_test_internal <- function(x, y, num.permutations = 99, distance = FALSE, we
       pvalue <- calculatePvalue(result[["statistic"]] * result[["info"]][["N"]], 
                                 BITestNullDistribution)
     } else {
-      if (weight == "none") {
+      if (weight == WEIGHT_TYPE[1]) {
         return(result[[1]][1])
-      } else if (weight == "prob") {
+      } else if (weight == WEIGHT_TYPE[2]) {
         return(result[[1]][2])
-      } else {
+      } else if (weight == WEIGHT_TYPE[3]) {
         return(result[[1]][3])
       }
     }
   } else {
     set.seed(seed = examine_seed_arguments(seed))
-    result <- bcov_test_wrap_c(x = x, y = y, n = num, num.permutations = num.permutations, distance =  distance, num.threads = num.threads)
+    result <- bcov_test_wrap_c(x = x, y = y, n = num, num.permutations = num.permutations, distance = distance, num.threads = num.threads)
     return(result)
   }
 }
@@ -274,17 +311,17 @@ bcov_test_internal <- function(x, y, num.permutations = 99, distance = FALSE, we
 #' @inherit return
 #' @noRd
 kbcov_test_internal <- function(x, num.permutations = 99, distance = FALSE, weight = FALSE, 
-                                seed = 4, method = 'permute', num.threads)
+                                seed = 1, method = 'permute', num.threads)
 {
-  x <- lapply(x, as.matrix)
-  size_list <- sapply(x, nrow)
-  num <- unique(size_list)
-  if(length(num) > 1) {
-    stop("sample sizes in each list must equal!")
-  }
   ############################################################
   #################### R Version (1.1.0) #####################
   ############################################################
+  # x <- lapply(x, as.matrix)
+  # size_list <- sapply(x, nrow)
+  # num <- unique(size_list)
+  # if(length(num) > 1) {
+    # stop("sample sizes of variables are not match!")
+  # }
   # if(distance) {
   #   
   # } else {
@@ -331,14 +368,30 @@ kbcov_test_internal <- function(x, num.permutations = 99, distance = FALSE, weig
   #################### C Version (1.2.0) #####################
   ############################################################
   var_num <- length(x)
+  if ((!distance) && all(sapply(x, class) == "dist")) {
+    distance <- TRUE
+  }
   if(distance) {
-    
+    if (class(x[[1]]) != "dist") {
+      if (nrow(x[[1]]) != ncol(x[[1]])) {
+        stop("The elements of input list is not a distance matrix.")
+      }
+    }
   } else {
-    x <- lapply(x, dist, diag = TRUE, upper = TRUE)
+    x <- lapply(x, dist)
   }
-  x <- lapply(x, as.matrix)
-  x <- unlist(lapply(x, as.vector))
   distance <- TRUE
+  num <- ifelse(class(x[[1]]) == "dist", attr(x[[1]], "Size"), nrow(x[[1]]))
+  if (class(x[[1]]) == "dist") {
+    x <- lapply(x, as.vector)
+  } else {
+    x <- lapply(x, function(xx) { xx[lower.tri(xx)] })
+  }
+  if (length(unique(sapply(x, length))) != 1) {
+    stop("sample size of variables are not match!")
+  } else {
+    x <- unlist(x)
+  }
   #
   if(num.permutations == 0) {
     result <- kbcov_test_wrap_c(x = x, K = var_num, n = num, num.permutations = 0, 
@@ -347,9 +400,9 @@ kbcov_test_internal <- function(x, num.permutations = 99, distance = FALSE, weig
       pvalue <- calculatePvalue(result[["statistic"]] * result[["info"]][["N"]], 
                                 BITestNullDistribution)
     } else {
-      if (weight == "none") {
+      if (weight == WEIGHT_TYPE[1]) {
         return(result[[1]][1])
-      } else if (weight == "prob") {
+      } else if (weight == WEIGHT_TYPE[2]) {
         return(result[[1]][2])
       } else {
         return(result[[1]][3])
@@ -371,9 +424,9 @@ kbcov_test_internal <- function(x, num.permutations = 99, distance = FALSE, weig
 #' @param num sample size
 #' @param var_num random variables number
 #' @param weight whether used weight
-#' @param type ball correlation or ball covariance. now, only ball covariance is considered.
+#' @param type Ball Correlation or Ball Covariance. now, only Ball Covariance is considered.
 #'
-#' @return ball covariance statistic
+#' @return Ball Covariance statistic
 #' @noRd
 kbcov_stat <- function(x, num, var_num, weight, type) {
   compare_list <- list()
@@ -425,7 +478,7 @@ kbcov_stat <- function(x, num, var_num, weight, type) {
   stat_value_prob <- stat_value_prob / (num)^2
   stat_value_hhg <- stat_value_hhg / (hhg_ball_num)^2
   # 
-  c("bcov" = stat_value, "bcov.prob" = stat_value_prob, "bcov.chisq" = stat_value_hhg)
+  c("bcov.constant" = stat_value, "bcov.probability" = stat_value_prob, "bcov.chisquare" = stat_value_hhg)
 }
 
 
@@ -456,92 +509,81 @@ bcov_test_internal_wrap <- function(x = x, y = y, num.permutations, distance, se
 }
 
 
-#' @title Ball Correlation and Covariance Statistics
-#' @description Computes ball covariance and ball correlation statistics, 
-#' which are multivariate measures of dependence in Banach space.
+#' @title Ball Covariance and Correlation Statistics
+#' @description Computes Ball Covariance and Ball Correlation statistics, 
+#' which are generic dependence measures in Banach spaces.
 #' @inheritParams bcov.test
 #' @rdname bcov
 #' 
 #' @details 
-#' \code{bcov} and \code{bcor} compute ball covariance and ball correlation statistics.
-#' 
-#' The sample sizes (number of rows or length of the vector) of the two variables must agree, 
-#' and samples must not contain missing values. 
+#' The sample sizes of the two variables must agree,  and samples must not contain missing and infinite values. 
 #' If we set \code{distance = TRUE}, arguments \code{x}, \code{y} can be a \code{dist} object or a
-#' symmetric numeric matrix recording distance between samples; 
-#' otherwise, these arguments are treated as data.
+#' symmetric numeric matrix recording distance between samples; otherwise, these arguments are treated as data.
 #' 
-#' Ball covariance is a generic non-parametric dependence measure in Banach space, introduced by Pan et al(2017). 
-#' It is noteworthy that ball covariance enjoys the following properties: 
+#' \code{bcov} and \code{bcor} compute Ball Covariance and Ball Correlation statistics.
 #' 
-#' (i) It is nonnegative, and holds the Cauchy-Schwartz type inequality; 
+#' Ball Covariance statistics is a generic dependence measure in Banach spaces. It enjoys the following properties: 
+#' \itemize{
+#' \item It is nonnegative and it is equal to zero if and only if variables are unassociated; 
+#' \item It is highly robust;   
+#' \item It is distribution-free and model-free;   
+#' \item it is interesting that the HHG is a special case of Ball Covariance statistics. 
+#' }
+#' Ball correlation statistics, a normalized version of Ball Covariance statistics, generalizes Pearson correlation in two fundamental ways: 
+#' \itemize{
+#' \item It is well-defined for random variables in arbitrary dimension in Banach spaces
+#' \item BCor is equal to zero implies random variables are unassociated.
+#' }
 #' 
-#' (ii) It is nonparametric and makes fewer restrictive data assumptions even without finite moment conditions;  
-#' 
-#' (iii) Its empirical version is feasible and can be used as a test statistic of independence with some desired test properties;  
-#' 
-#' (iv) it is interesting that the HHG dependence measure is a special case of ball covariance. 
-#' 
-#' Ball correlation, based on the normalized ball covariance, generalizes the idea of Pearson correlation in two fundamental ways: 
-#' 
-#' (i) Ball correlation, \eqn{ \mathbf{BCor}_{\omega}^{2}(X, Y) }, is defined for \eqn{X} and \eqn{Y} in arbitrary dimension in Banach space. 
-#' 
-#' (ii) Ball correlation satisfies \eqn{0 \le \mathbf{BCor}_{\omega}^{2}(X, Y) \le 1}, and \eqn{ \mathbf{BCor}_{\omega}^{2}(X, Y) } = 0 
-#' only if \eqn{X} and \eqn{Y} are independent.
-#' 
-#' The definitions of the sample version ball covariance and ball correlation are as follows.
+#' The definitions of the Ball Covariance and Ball Correlation statistics between two random variables are as follows.
 #' Suppose, we are given pairs of independent observations 
 #' \eqn{\{(x_1, y_1),...,(x_n,y_n)\}}, where \eqn{x_i} and \eqn{y_i} can be of any dimension 
 #' and the dimensionality of \eqn{x_i} and \eqn{y_i} need not be the same.
-#' Then, we define sample version ball covariance as:
-#' \deqn{\mathbf{BCor}_{\omega, n}^{2}(X, Y)=\frac{1}{n^{2}}\sum_{i,j=1}^{n}{(\Delta_{ij,n}^{X,Y}-\Delta_{ij,n}^{X}\Delta_{ij,n}^{Y})^{2}} }
+#' Then, we define sample version Ball Covariance as:
+#' \deqn{\mathbf{BCov}_{\omega, n}^{2}(X, Y)=\frac{1}{n^{2}}\sum_{i,j=1}^{n}{(\Delta_{ij,n}^{X,Y}-\Delta_{ij,n}^{X}\Delta_{ij,n}^{Y})^{2}} }
 #' where:
 #' \deqn{ \Delta_{ij,n}^{X,Y}=\frac{1}{n}\sum_{k=1}^{n}{\delta_{ij,k}^{X} \delta_{ij,k}^{Y}}, 
 #' \Delta_{ij,n}^{X}=\frac{1}{n}\sum_{k=1}^{n}{\delta_{ij,k}^{X}}, 
 #' \Delta_{ij,n}^{Y}=\frac{1}{n}\sum_{k=1}^{n}{\delta_{ij,k}^{Y}} }
 #' \deqn{\delta_{ij,k}^{X} = I(x_{k} \in \bar{B}(x_{i}, \rho(x_{i}, x_{j}))), 
 #' \delta_{ij,k}^{Y} = I(y_{k} \in \bar{B}(y_{i}, \rho(y_{i}, y_{j})))}
-#' 
 #' Among them, \eqn{\bar{B}(x_{i}, \rho(x_{i}, x_{j}))} is a closed ball 
 #' with center \eqn{x_{i}} and radius \eqn{\rho(x_{i}, x_{j})}.
-#' Similarly, we can give the notations \eqn{ \mathbf{BCov}_{\omega,n}^2(\mathbf{X},\mathbf{X}) } 
-#' and \eqn{ \mathbf{BCov}_{\omega,n}^2(\mathbf{Y},\mathbf{Y}) }, 
-#' which are the sample version of \eqn{ \mathbf{BCov}_{\omega}^2(\mathbf{X},\mathbf{X}) } and 
-#' \eqn{ \mathbf{BCov}_{\omega}^2(\mathbf{Y},\mathbf{Y}) }. 
-#' We thus define the sample version ball correlation as follows.
-#' 
+#' Similarly, we can define \eqn{ \mathbf{BCov}_{\omega,n}^2(\mathbf{X},\mathbf{X}) } 
+#' and \eqn{ \mathbf{BCov}_{\omega,n}^2(\mathbf{Y},\mathbf{Y}) }. 
+#' We define Ball Correlation statistic as follows.
 #' \deqn{\mathbf{BCor}_{\omega,n}^2(\mathbf{X},\mathbf{Y})=
 #' \mathbf{BCov}_{\omega,n}^2(\mathbf{X},\mathbf{Y})/\sqrt{\mathbf{BCov}_{\omega,n}^2(\mathbf{X},\mathbf{X})\mathbf{BCov}_{\omega,n}^2(\mathbf{Y},\mathbf{Y})}
 #' }
 #' 
-#' Moreover, it is natural to extend \eqn{\mathbf{BCov}_{\omega,n}} to measure the mutual independence between \eqn{K} random variables:
+#' We can extend \eqn{\mathbf{BCov}_{\omega,n}} to measure the mutual independence between \eqn{K} random variables:
+#' \deqn{\frac{1}{n^{2}}\sum_{i,j=1}^{n}{\left[ (\Delta_{ij,n}^{X_{1}, ..., X_{K}}-\prod_{k=1}^{K}\Delta_{ij,n}^{X_{k}})^{2}\prod_{k=1}^{K}{\hat{\omega}_{k}(X_{ki},X_{kj})} \right]}}
+#' where \eqn{X_{k}(k=1,\ldots,K)} are random variables and \eqn{X_{ki}} is the \eqn{i}-th observations of \eqn{X_{k}}. 
 #' 
-#' \deqn{\frac{1}{n^{2}}\sum_{i,j=1}^{n}{\left[ (\Delta_{ij,n}^{R_{1}, ..., R_{K}}-\prod_{k=1}^{K}\Delta_{ij,n}^{R_{k}})^{2}\prod_{k=1}^{K}{\hat{\omega}_{k}(R_{ki},R_{kj})} \right]}}
-#' 
-#' where \eqn{R_{k}, k=1,...K} indicate random variables and \eqn{R_{ki}, i=1,...,n} denote \eqn{i} th random samples of \eqn{R_{k}}. 
-#' 
-#' See \code{\link{bcov.test}} for a test of multivariate independence based on the 
-#' ball covariance and ball correlation statistic.
+#' See \code{\link{bcov.test}} for a test of independence based on the Ball Covariance statistic.
 #' 
 #' @return 
-#' \item{\code{bcov }}{ sample version of ball covariance.}
+#' \item{\code{bcov }}{ Ball Covariance statistic.}
 #' @seealso
 #' \code{\link{bcov.test}}, \code{\link{bcorsis}}
 #' @export
 #' 
+#' @references Wenliang Pan, Xueqin Wang, Heping Zhang, Hongtu Zhu & Jin Zhu (2019) Ball Covariance: A Generic Measure of Dependence in Banach Space, Journal of the American Statistical Association, DOI: 10.1080/01621459.2018.1543600
+#' @references Wenliang Pan, Xueqin Wang, Weinan Xiao & Hongtu Zhu (2018) A Generic Sure Independence Screening Procedure, Journal of the American Statistical Association, DOI: 10.1080/01621459.2018.1462709
+#' @references Jin Zhu, Wenliang Pan, Wei Zheng, and Xueqin Wang (2018). Ball: An R package for detecting distribution difference and association in metric spaces. arXiv preprint arXiv:1811.03750. http://arxiv.org/abs/1811.03750
+#' 
 #' @examples
 #' ############# Ball Covariance #############
-#' n <- 50
-#' x <- rnorm(n)
-#' y <- rnorm(n)
+#' num <- 50
+#' x <- rnorm(num)
+#' y <- rnorm(num)
 #' bcov(x, y)
-#' bcov(x, y, weight = TRUE)
 #' bcov(x, y, weight = "prob")
 #' bcov(x, y, weight = "chisq")
 bcov <- function(x, y, distance = FALSE, weight = FALSE) {
   weight <- examine_weight_arguments(weight)
-  res <- bcov_test_internal_wrap(x = x, y = y, num.permutations = 0, distance = distance, seed = 0,
-                                 weight = weight, method = "permute", num.threads = 1)
+  res <- bcov_test_internal_wrap(x = x, y = y, num.permutations = 0, distance = distance, seed = 1,
+                                 weight = weight, method = "permute", num.threads = 0)
   res
 }
 
