@@ -41,11 +41,11 @@ bcov.test(x = x, y = y)
 
 ## ------------------------------------------------------------------------
 x <- matrix(runif(50 * 2, -pi, pi), nrow = 50, ncol = 2)
-error <- runif(50, min = -0.3, max = 0.3)
-y <- (sin((x[,1])^2 + x[,2])) + error
+error <- runif(50, min = -0.1, max = 0.1)
+y <- 2 * sin(x[,1] + x[,2]) + error
 
 ## ------------------------------------------------------------------------
-bcov.test(x = x, y = y)
+bcov.test(x = x, y = y, weight = "prob")
 
 ## ------------------------------------------------------------------------
 # load data:
@@ -58,10 +58,10 @@ data("bdvmf")
 
 ## ------------------------------------------------------------------------
 # calculate geodesic distance between samples:
-Dmat <- nhdist(bdvmf[["x"]], method = "geodesic")
+dx <- nhdist(bdvmf[["x"]], method = "geodesic")
 # sample sizes in each group: 150, 150
 # Two-Sample Test based on BD :
-bd.test(x = Dmat, size = c(150, 150), R = 99, dst = TRUE)
+bd.test(x = dx, size = c(150, 150), num.permutations = 99, distance = TRUE)
 
 ## ------------------------------------------------------------------------
 # load data:
@@ -69,18 +69,18 @@ data("macaques")
 # number of femala and male Macaca fascicularis:
 # table(macaques[["group"]])  # f: 9; m: 9
 # calculate Riemannian shape distance matrix:
-Dmat <- nhdist(macaques[["x"]], method = "riemann")
+dx <- nhdist(macaques[["x"]], method = "riemann")
 # hypothesis test with BD:
-bd.test(x = Dmat, R = 99, size = c(9, 9), dst = TRUE)
+bd.test(x = dx, num.permutations = 99, size = c(9, 9), distance = TRUE)
 
 ## ------------------------------------------------------------------------
 data("ArcticLake")
 # Distance matrix between y:
-Dy <- nhdist(ArcticLake[["x"]], method = "compositional")
+dy <- nhdist(ArcticLake[["x"]], method = "compositional")
 # Distance matrix between x:
-Dx <- dist(ArcticLake[["depth"]])
+dx <- dist(ArcticLake[["depth"]])
 # hypothesis test with BCov:
-bcov.test(x = Dx, y = Dy, R = 99, dst = TRUE)
+bcov.test(x = dx, y = dy, num.permutations = 99, distance = TRUE)
 
 ## ------------------------------------------------------------------------
 n <- 150
@@ -91,17 +91,17 @@ data("ArcticLake")
 Dy <- nhdist(ArcticLake[["x"]], method = "compositional")
 Dx <- dist(ArcticLake[["depth"]])
 # hypothesis test with weighted BCov:
-bcov.test(x = Dx, y = Dy, R = 99, 
-          dst = TRUE, weight = TRUE)
+bcov.test(x = Dx, y = Dy, num.permutations = 99, 
+          distance = TRUE, weight = "constant")
 
 ## ------------------------------------------------------------------------
-x <- rnorm(30)
-y <- (x > 0) * x + rnorm(30)
-z <- (x <= 0) * x + rnorm(30)
+x <- rnorm(50)
+y <- (x > 0) * x + rnorm(50)
+z <- (x <= 0) * x + rnorm(50)
 example1 <- list(x, y, z)
 
 ## ------------------------------------------------------------------------
-h <- rnorm(30)
+h <- rnorm(50)
 w <- (h)^2
 x <- abs(h)
 y <- h * (h < 0)
@@ -111,8 +111,8 @@ z <- cbind(z1, z2)
 example2 <- list(w, x, y, z)
 
 ## ------------------------------------------------------------------------
-bcov.test(x = example1)
-bcov.test(x = example2)
+bcov.test(x = example1, num.permutations = 199)
+bcov.test(x = example2, num.permutations = 199)
 
 ## ------------------------------------------------------------------------
 set.seed(1)
